@@ -1,6 +1,6 @@
 ;;; context.el --- Support for ConTeXt documents.
 
-;; Copyright (C) 2003-2006, 2008, 2010, 2012, 2014
+;; Copyright (C) 2003-2006, 2008, 2010, 2012, 2014-2017
 ;;   Free Software Foundation, Inc.
 
 ;; Maintainer: Berend de Boer <berend@pobox.com>
@@ -64,16 +64,19 @@
 
 ;;; variables
 
-;; globals used in certain macro's.
-(defvar done-mark nil
-  "Position of point afterwards, default nil (meaning end).")
+;; Dynamically scoped vars used in certain macro's.
+;; BEWARE: We used to give them a global nil value, but this can mess up poor
+;; unrelated packages using those same vars but expecting them to be
+;; lexically scoped.
+;; So don't give them a global value, which makes sure the effect of `defvar'
+;; localized to this file!
+(defvar done-mark)     ;Position of point afterwards, default nil (meaning end)
 
-(defvar reference nil
-  "Set by `ConTeXt-section-ref', used by `ConTeXt-section-section'.")
-
-(defvar title nil
-  "Set by `ConTeXt-section-title', used by `ConTeXt-section-section'.")
+(defvar reference);Used by `ConTeXt-section-ref' and `ConTeXt-section-section'.
 
+(defvar title); Used by `ConTeXt-section-title' and `ConTeXt-section-section'.
+(defvar name)
+(defvar level)
 
 ;; others
 
@@ -302,8 +305,8 @@ The following variables can be set to customize:
 		       (ConTeXt-up-section (- val)))
 		      (t val)))
 	 (name (ConTeXt-numbered-section-name level))
-	 (toc nil)
 	 (title "")
+         (reference nil)
 	 (done-mark (make-marker)))
     (newline)
     (run-hooks 'ConTeXt-numbered-section-hook)
@@ -410,9 +413,9 @@ section."
 
 The following variables are set before the hooks are run
 
-level - numeric section level, see the documentation of `ConTeXt-section'.
-name - name of the sectioning command, derived from `level'.
-title - The title of the section, default to an empty string.
+`level' - numeric section level, see the documentation of `ConTeXt-section'.
+`name' - name of the sectioning command, derived from `level'.
+`title' - The title of the section, default to an empty string.
 `done-mark' - Position of point afterwards, default nil (meaning end).
 
 The following standard hook exist -
@@ -451,13 +454,14 @@ in your .emacs file."
     ConTeXt-section-title
     ConTeXt-section-ref
     ConTeXt-section-section)
+  ;; FIXME: I can't see where this variable is used!
   "List of hooks to run when a new section is inserted.
 
 The following variables are set before the hooks are run
 
-level - numeric section level, see the documentation of `ConTeXt-section'.
-name - name of the sectioning command, derived from `level'.
-title - The title of the section, default to an empty string.
+`level' - numeric section level, see the documentation of `ConTeXt-section'.
+`name' - name of the sectioning command, derived from `level'.
+`title' - The title of the section, default to an empty string.
 `done-mark' - Position of point afterwards, default nil (meaning end).
 
 The following standard hook exist -
@@ -491,6 +495,14 @@ in your .emacs file."
     ConTeXt-section-ref
     ConTeXt-section-section))
 
+;; Define before first use.
+(defcustom ConTeXt-Mark-version "II"
+  "ConTeXt Mark version used for running ConTeXt."
+  :type 'string
+  :group 'TeX-command)
+(make-variable-buffer-local 'ConTeXt-Mark-version)
+(put 'ConTeXt-Mark-version 'safe-local-variable 'stringp)
+
 (defun ConTeXt-numbered-section-heading ()
   "Hook to prompt for ConTeXt section name.
 Insert this hook into `ConTeXt-numbered-section-hook' to allow the user to change
@@ -519,7 +531,7 @@ the name of the sectioning command inserted with `\\[ConTeXt-section]'."
   "Hook to prompt for ConTeXt section title.
 Insert this hook into `ConTeXt-(un)numbered-section-hook' to allow the user to change
 the title of the section inserted with `\\[ConTeXt-section]."
-  (setq title (read-string "What title: ")))
+  (setq title (TeX-read-string "What title: ")))
 
 (defun ConTeXt-section-section ()
   "Hook to insert ConTeXt section command into the file.
@@ -560,28 +572,48 @@ inserted after the sectioning command."
 ;; Various
 (defun TeX-ConTeXt-sentinel (process name)
   "Cleanup TeX output buffer after running ConTeXt."
-  (cond ((TeX-TeX-sentinel-check process name))
-	((save-excursion
-	   ;; in a full ConTeXt run there will multiple texutil
-	   ;; outputs. Just looking for "another run needed" would
-	   ;; find the first occurence
-	   (goto-char (point-max))
-	   (re-search-backward "TeXUtil " nil t)
-	   (re-search-forward "another run needed" nil t))
-	 (message (concat "You should run ConTeXt again "
-			  "to get references right, "
-			  (TeX-current-pages)))
-	 (setq TeX-command-next TeX-command-default))
-	((re-search-forward "removed files :" nil t)
-	 (message "sucessfully cleaned up"))
-	((re-search-forward "^ ?TeX\\(Exec\\|Util\\)" nil t) ;; strange regexp --pg
-	 (message (concat name ": successfully formatted "
-			  (TeX-current-pages)))
-	 (setq TeX-command-next TeX-command-Show))
-	(t
-	 (message (concat name ": problems after "
-			  (TeX-current-pages)))
-	 (setq TeX-command-next TeX-command-default))))
+  (cond
+   ;; Mark IV
+   ((with-current-buffer TeX-command-buffer
+      (string= ConTeXt-Mark-version "IV"))
+    (cond ((TeX-TeX-sentinel-check process name))
+	  ((re-search-forward "fatal error: " nil t)
+	   (message (concat name ": problems after "
+			    (TeX-current-pages)))
+	   (setq TeX-command-next TeX-command-default))
+	  (t
+	   (message (concat name ": successfully formatted "
+			    (TeX-current-pages)))
+	   (setq TeX-command-next TeX-command-Show))))
+   ;; Mark II
+   (t
+    (cond ((TeX-TeX-sentinel-check process name))
+	  ((save-excursion
+	     ;; in a full ConTeXt run there will multiple texutil
+	     ;; outputs. Just looking for "another run needed" would
+	     ;; find the first occurence
+	     (goto-char (point-max))
+	     (re-search-backward "TeXUtil " nil t)
+	     (re-search-forward "another run needed" nil t))
+	   (message (concat "You should run ConTeXt again "
+			    "to get references right, "
+			    (TeX-current-pages)))
+	   (setq TeX-command-next TeX-command-default))
+	  ((re-search-forward "removed files :" nil t)
+	   (message "sucessfully cleaned up"))
+	  ((re-search-forward "^ ?TeX\\(Exec\\|Util\\)" nil t) ;; strange regexp --pg
+	   (message (concat name ": successfully formatted "
+			    (TeX-current-pages)))
+	   (setq TeX-command-next TeX-command-Show))
+	  (t
+	   (message (concat name ": problems after "
+			    (TeX-current-pages)))
+	   (setq TeX-command-next TeX-command-default)))))
+  (unless TeX-error-list
+    (run-hook-with-args 'TeX-after-compilation-finished-functions
+			(with-current-buffer TeX-command-buffer
+			  (expand-file-name
+			   (TeX-active-master (TeX-output-extension)))))))
 
 
 ;;; Environments
@@ -630,33 +662,26 @@ inserted after the sectioning command."
 	 ;; this should not happen
 	 (error "Unknown interface: %s" ConTeXt-current-interface))))
 
-
 (defun ConTeXt-environment (arg)
   "Make ConTeXt environment (\\start...-\\stop... pair).
 With optional ARG, modify current environment."
   (interactive "*P")
-  (let ((environment (
-		      completing-read (concat "Environment type: (default "
-					      (if (TeX-near-bobp)
-						  "text"
-						ConTeXt-default-environment)
-					      ") ")
-		      ConTeXt-environment-list
-		      nil nil nil
-		      'ConTeXt-environment-history)
-		     ))
-    ;; Get default
-    (cond ((and (zerop (length environment))
-		(TeX-near-bobp))
-	   (setq environment "text"))
-	  ((zerop (length environment))
-	   (setq environment ConTeXt-default-environment))
-	  (t
-	   (setq ConTeXt-default-environment environment)))
+  (let* ((default (cond
+		   ((TeX-near-bobp) "text")
+		   (t ConTeXt-default-environment)))
+	 (environment
+	  (completing-read (concat "Environment type: (default " default ") ")
+			   ConTeXt-environment-list nil nil nil
+			   'ConTeXt-environment-history default)))
+    ;; Use `environment' as default for the next time only if it is different
+    ;; from the current default.
+    (unless (equal environment default)
+      (setq ConTeXt-default-environment environment))
 
     (let ((entry (assoc environment ConTeXt-environment-list)))
-      (when (null entry)
-	(ConTeXt-add-environments (list environment)))
+      (if (null entry)
+	  (ConTeXt-add-environments (list environment)))
+
       (if arg
 	  (ConTeXt-modify-environment environment)
 	(ConTeXt-environment-menu environment)))))
@@ -856,8 +881,7 @@ If INNER is non-nil, go to the point just past before
 			(ConTeXt-environment-stop-name)
 			"\\)"
 			))
-	(level 1)
-	(pos))
+	(level 1))
     ;;jump over the \start... when at the beginning of it.
     (when (looking-at (concat (regexp-quote TeX-esc)
 			      (ConTeXt-environment-start-name)))
@@ -935,13 +959,13 @@ If INNER is non-nil, go to the point just past the \\start... macro."
 
 ;;; Macro Argument Hooks
 
-(defun ConTeXt-optional-argument-insert (arg &optional prefix)
+(defun ConTeXt-optional-argument-insert (arg &optional _prefix)
   "Insert ARG surrounded by square brackets."
   (insert ConTeXt-optop)
   (insert arg)
   (insert ConTeXt-optcl))
 
-(defun ConTeXt-required-argument-insert (arg &optional prefix)
+(defun ConTeXt-required-argument-insert (arg &optional _prefix)
   "Insert ARG surrounded by curly braces."
   (insert TeX-grop)
   (insert arg)
@@ -988,11 +1012,11 @@ If OPTIONAL, only insert it if not empty, and then use square brackets."
    "[][]\\|"  ; display math delimitors (is this applicable to ConTeXt??)
    (ConTeXt-environment-start-name) "\\|"
    (ConTeXt-environment-stop-name) "\\|"
-   (mapconcat 'car ConTeXt-numbered-section-list "\\b\\|") "\\b\\|"
-   (mapconcat 'car ConTeXt-unnumbered-section-list "\\b\\|") "\\b\\|"
-   (mapconcat 'identity ConTeXt-extra-paragraph-commands "\\b\\|")
+   (mapconcat #'car ConTeXt-numbered-section-list "\\b\\|") "\\b\\|"
+   (mapconcat #'car ConTeXt-unnumbered-section-list "\\b\\|") "\\b\\|"
+   (mapconcat #'identity ConTeXt-extra-paragraph-commands "\\b\\|")
    "\\b\\|"
-   (mapconcat 'identity ConTeXt-item-list "\\b\\|") "\\b\\)"))
+   (mapconcat #'identity ConTeXt-item-list "\\b\\|") "\\b\\)"))
 
 
 ;; Outline support
@@ -1012,14 +1036,14 @@ header is at the start of a line."
    "[ \t]*"
    (regexp-quote TeX-esc)
    "\\("
-   (mapconcat 'ConTeXt-environment-full-start-name ConTeXt-section-block-list "\\|") "\\|"
-   (mapconcat 'car ConTeXt-numbered-section-list "\\|")
-   (mapconcat 'car ConTeXt-unnumbered-section-list "\\|")
+   (mapconcat #'ConTeXt-environment-full-start-name ConTeXt-section-block-list "\\|") "\\|"
+   (mapconcat #'car ConTeXt-numbered-section-list "\\|") "\\|"
+   (mapconcat #'car ConTeXt-unnumbered-section-list "\\|")
    "\\)\\b"
    (if TeX-outline-extra
        "\\|"
      "")
-   (mapconcat 'car TeX-outline-extra "\\|")
+   (mapconcat #'car TeX-outline-extra "\\|")
    "\\|" (ConTeXt-header-end) "\\b"
    "\\|" (ConTeXt-trailer-start) "\\b"))
 
@@ -1047,7 +1071,7 @@ header is at the start of a line."
   "Regular expression that matches a start of all environments mentioned in LIST."
   (concat
    "start\\("
-   (mapconcat 'identity list "\\|")
+   (mapconcat #'identity list "\\|")
    "\\)\\b"))
 
 ;; The top headings are \starttext, \startfrontmatter, \startbodymatter etc.
@@ -1150,7 +1174,7 @@ An optional fourth (or sixth) element means always replace if t."
   (modify-syntax-entry ?\( "." ConTeXt-indent-syntax-table)
   (modify-syntax-entry ?\) "." ConTeXt-indent-syntax-table))
 
-(defun ConTeXt-indent-line (&optional arg)
+(defun ConTeXt-indent-line (&optional _arg)
   (with-syntax-table ConTeXt-indent-syntax-table
     ;; TODO: Rather than ignore $, we should try to be more clever about it.
     (let ((indent
@@ -1196,7 +1220,10 @@ else.  There might be text before point."
        (condition-case err
 	   (progn
 	     (backward-sexp 1)
-	     (while (> (current-column) (current-indentation))
+	     (while (or (> (current-column) (current-indentation))
+			;; Continue going back if we are
+			;; at a hanging optional group.
+			(looking-at (regexp-quote ConTeXt-optop)))
 	       (backward-sexp 1)))
 	 (scan-error
 	  (setq up-list-pos (nth 2 err))))
@@ -1517,55 +1544,55 @@ else.  There might be text before point."
 	(TeX-update-style)
 	(setq ConTeXt-menu-changed nil)
 	(message "Updating section menu...")
-	(mapc 'ConTeXt-section-enable ConTeXt-section-list)
+	(mapc #'ConTeXt-section-enable ConTeXt-section-list)
 	(message "Updating environment menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-environment-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-environment-menu-entry
+			   (mapcar #'ConTeXt-environment-menu-entry
 				   (ConTeXt-environment-list))))
 	(message "Updating modify environment menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-environment-modify-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-environment-modify-menu-entry
+			   (mapcar #'ConTeXt-environment-modify-menu-entry
 				   (ConTeXt-environment-list))))
 	(message "Updating define menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-define-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-define-menu-entry
+			   (mapcar #'ConTeXt-define-menu-entry
 				   ConTeXt-define-list)))
 	(message "Updating setup menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-setup-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-setup-menu-entry
+			   (mapcar #'ConTeXt-setup-menu-entry
 				   ConTeXt-setup-list)))
 	(message "Updating referencing menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-referencing-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-referencing-menu-entry
+			   (mapcar #'ConTeXt-referencing-menu-entry
 				   ConTeXt-referencing-list)))
 	(message "Updating other macro's menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-other-macro-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-other-macro-menu-entry
+			   (mapcar #'ConTeXt-other-macro-menu-entry
 				   ConTeXt-other-macro-list)))
 	(message "Updating project structure menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-project-structure-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-project-structure-menu-entry
+			   (mapcar #'ConTeXt-project-structure-menu-entry
 				   ConTeXt-project-structure-list)))
 	(message "Updating section block menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-section-block-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-section-block-menu-entry
+			   (mapcar #'ConTeXt-section-block-menu-entry
 				   ConTeXt-section-block-list)))
 	(message "Updating section menu...")
 	(easy-menu-change '("ConTeXt") ConTeXt-numbered-section-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-numbered-section-menu-entry
+			   (mapcar #'ConTeXt-numbered-section-menu-entry
 				   ConTeXt-numbered-section-list)))
 	(easy-menu-change '("ConTeXt") ConTeXt-unnumbered-section-menu-name
 			  (LaTeX-split-long-menu
-			   (mapcar 'ConTeXt-unnumbered-section-menu-entry
+			   (mapcar #'ConTeXt-unnumbered-section-menu-entry
 				   ConTeXt-unnumbered-section-list)))
 	(message "Updating...done")
 	(and menu (easy-menu-return-item ConTeXt-mode-menu menu))
@@ -1576,21 +1603,41 @@ else.  There might be text before point."
 (defvar ConTeXt-texexec-option-nonstop "--nonstop "
   "Command line option for texexec to use nonstopmode.")
 
+(defun ConTeXt-expand-command ()
+  "Expand ConTeXt command.
+Use `ConTeXt-Mark-version' to choose the command."
+  (cond
+   ((string= ConTeXt-Mark-version "IV")
+    "context")
+   ;; In any other case fall back on Mark II.
+   (t
+    "texexec")))
+
 (defun ConTeXt-expand-options ()
   "Expand options for context command."
-  (concat
-   (let ((engine (eval (nth 4 (assq TeX-engine (TeX-engine-alist))))))
-     (when engine
-       (format "--engine=%s " engine)))
-   (unless (eq ConTeXt-current-interface "en")
-     (format "--interface=%s " ConTeXt-current-interface))
-   (when TeX-source-correlate-mode
-     (format "--passon=\"%s\" "
-	     (if (eq (TeX-source-correlate-method-active) 'synctex)
-		 TeX-synctex-tex-flags
-	       TeX-source-specials-tex-flags)))
-   (unless TeX-interactive-mode
-     ConTeXt-texexec-option-nonstop)))
+  (cond
+   ;; Mark IV
+   ((string= ConTeXt-Mark-version "IV")
+    (concat
+     (if TeX-source-correlate-mode
+	 "--synctex=1 ")
+     (unless TeX-interactive-mode
+       ConTeXt-texexec-option-nonstop)))
+   ;; In any other case fall back on Mark II.
+   (t
+    (concat
+     (let ((engine (eval (nth 4 (TeX-engine-in-engine-alist TeX-engine)))))
+       (when engine
+	 (format "--engine=%s " engine)))
+     (unless (eq ConTeXt-current-interface "en")
+       (format "--interface=%s " ConTeXt-current-interface))
+     (when TeX-source-correlate-mode
+       (format "--passon=\"%s\" "
+	       (if (eq (TeX-source-correlate-method-active) 'synctex)
+		   TeX-synctex-tex-flags
+		 TeX-source-specials-tex-flags)))
+     (unless TeX-interactive-mode
+       ConTeXt-texexec-option-nonstop)))))
 
 ;;; Mode
 
@@ -1609,7 +1656,15 @@ else.  There might be text before point."
     ConTeXt-section-list
     ConTeXt-text
     ConTeXt-item-list
-    ConTeXt-extra-paragraph-commands))
+    ConTeXt-extra-paragraph-commands
+    ConTeXt-environment-list)
+  "List of variables to be set from languages specific ones.")
+
+(defconst ConTeXt-dialect :context
+  "Default dialect for use with function `TeX-add-style-hook' for
+argument DIALECT-EXPR when the hook is to be run only on ConTeXt
+file, or any mode derived thereof. See variable
+`TeX-style-hook-dialect'." )
 
 (defcustom ConTeXt-clean-intermediate-suffixes
   ;; See *suffixes in texutil.pl.
@@ -1645,6 +1700,7 @@ i.e. you do _not_ have to cater for this yourself by adding \\\\' or $."
   (setq major-mode 'context-mode)
 
   (setq local-abbrev-table context-mode-abbrev-table)
+  (set (make-local-variable 'TeX-style-hook-dialect) ConTeXt-dialect)
 
   ;; Make language specific variables buffer local
   (dolist (symbol ConTeXt-language-variable-list)
@@ -1656,7 +1712,7 @@ i.e. you do _not_ have to cater for this yourself by adding \\\\' or $."
 					      ConTeXt-current-interface)))))
 
   ;; Create certain regular expressions based on language
-  (setq ConTeXt-indent-item-re (concat "\\\\\\(" (mapconcat 'identity ConTeXt-item-list "\\|") "\\)\\>"))
+  (setq ConTeXt-indent-item-re (concat "\\\\\\(" (mapconcat #'identity ConTeXt-item-list "\\|") "\\)\\>"))
 
   ;; What's the deepest level at we can collapse a document?
   ;; set only if user has not set it. Need to be set before menu is created.
@@ -1696,7 +1752,7 @@ i.e. you do _not_ have to cater for this yourself by adding \\\\' or $."
   (easy-menu-add ConTeXt-mode-command-menu ConTeXt-mode-map)
   (setq ConTeXt-menu-changed t)
 
-  (if (= emacs-major-version 20)
+  (if (fboundp 'make-local-hook)
       (make-local-hook 'activate-menubar-hook))
   (add-hook 'activate-menubar-hook 'ConTeXt-menu-update nil t)
 
