@@ -5,7 +5,8 @@
 (TeX-add-style-hook "amsbook"
  (function
   (lambda ()
-    (TeX-run-style-hooks "amsmath" "amsthm")))
+    (TeX-run-style-hooks "amsmath" "amsthm")
+    (LaTeX-add-environments "abstract")))
  LaTeX-dialect)
 
 ;;; amsbook.el ends here.
