@@ -37,7 +37,8 @@
    ;; Fontification
    (when (and (featurep 'font-latex)
 	      (eq TeX-install-font-lock 'font-latex-setup))
-     (font-latex-add-keywords '(("nicefrac" "[{{")) 'textual))))
+     (font-latex-add-keywords '(("nicefrac" "[{{")) 'textual)))
+ LaTeX-dialect)
 
 (defvar LaTeX-nicefrac-package-options '("nice" "ugly")
   "Package options for the nicefrac package.")
