@@ -1,6 +1,6 @@
 ;;; tex-info.el --- Support for editing Texinfo source.
 
-;; Copyright (C) 1993, 1994, 1997, 2000, 2001, 2004, 2005, 2006, 2011
+;; Copyright (C) 1993, 1994, 1997, 2000, 2001, 2004, 2005, 2006, 2011-2015
 ;;   Free Software Foundation, Inc.
 
 ;; Maintainer: auctex-devel@gnu.org
@@ -25,6 +25,8 @@
 
 ;;; Code:
 
+(eval-when-compile (require 'cl))
+
 (require 'tex)
 
 (require 'texinfo)
@@ -288,6 +290,63 @@ beginning of keyword `@node' or `@bye'."
       (goto-char beg)
       (TeX-activate-region) )))
 
+(defun Texinfo-nodename-de-escape (node-name)
+  "In NODE-NAME, convert `@comma{}' commands to the corresponding `,'
+character. Return the resulting string."
+  (let ((pos 0) (map '(("comma" . ","))))
+    (while (and (< pos (length
+			node-name)) (string-match "@\\(comma\\)[[:blank:]]*{}" node-name pos))
+      (setq node-name (concat  (substring node-name 0 (match-beginning 0))
+			       (cdr (TeX-assoc-string (match-string 1 node-name) map))
+			       (substring node-name (match-end 0)))
+	    pos (1+ (match-beginning 0)))))
+  node-name)
+
+
+(defun Texinfo-nodename-escape (node-name)
+  "Convert in NODE-NAME the `,' characters to `@comma{}'
+commands. Return the resulting string."
+  (let* ((pos 0)
+	 (map '(("," . "comma")))
+	 (re (regexp-opt (mapcar 'car map))) )
+    (while (and (< pos (length node-name)) (string-match re node-name pos))
+      (setq node-name (concat  (substring node-name 0 (match-beginning 0))
+			       "@" (cdr (TeX-assoc-string (match-string 0 node-name) map))
+			       "{}"
+			       (substring node-name (match-end 0)))
+	    pos (1+ (match-beginning 0)))))
+  node-name)
+
+
+(defun Texinfo-make-node-list ()
+  ;; Build list of nodes in current buffer.
+  ;; (What about using `imenu--index-alist'?)
+  ;; FIXME: Support multi-file documents.
+  (save-excursion
+    (goto-char (point-min))
+    (let (nodes dups)
+      (while (re-search-forward "^@node\\b" nil t)
+	(skip-chars-forward "[:blank:]")
+	(pushnew (list (Texinfo-nodename-de-escape
+			(buffer-substring-no-properties
+			 (point) (progn (skip-chars-forward "^\r\n,")
+					(skip-chars-backward "[:blank:]")
+					(point)))))
+		 nodes
+		 :test (lambda (a b)
+			 (when (equal a b)
+			   (push (cons a (TeX-line-number-at-pos (point))) dups)
+			   t))))
+      (when dups
+	(display-warning
+	 'AUCTeX
+	 (format "There are duplicate nodes:\n%s"
+		 (mapconcat (lambda (dup)
+			      (format "    %s on line %d" (car dup) (cdr dup)))
+			    (nreverse dups)
+			    "\n"))))
+      (nreverse nodes))))
+
 (defun Texinfo-insert-node ()
   "Insert a Texinfo node in the current buffer.
 That means, insert the string `@node' and prompt for current,
@@ -297,26 +356,20 @@ a comment on the following line indicating the order of arguments
 for @node."
   (interactive)
   (let ((active-mark (and (TeX-active-mark) (not (eq (mark) (point)))))
-	nodes node-name next-node previous-node up-node)
-    ;; Build list of nodes in current buffer.
-    ;; (What about using `imenu--index-alist'?)
-    ;; FIXME: Support multi-file documents.
-    (save-excursion
-      (goto-char (point-min))
-      (while (re-search-forward "^@node\\b" nil t)
-	(skip-chars-forward " \t")
-	(add-to-list 'nodes
-		     (list (buffer-substring-no-properties
-			    (point) (progn (skip-chars-forward "^,")
-					   (point)))))))
+	(nodes (Texinfo-make-node-list))
+	node-name next-node previous-node up-node)
     (unless active-mark
-      (setq node-name (read-string "Node name: ")))
+      (setq node-name (Texinfo-nodename-escape
+		       (TeX-read-string "Node name: "))))
     ;; FIXME: What if key binding for `minibuffer-complete' was changed?
     ;; `substitute-command-keys' doesn't return the correct value.
-    (setq next-node (completing-read "Next node (TAB completes): " nodes))
+    (setq next-node (Texinfo-nodename-escape
+		     (completing-read "Next node (TAB completes): " nodes)))
     (setq previous-node
-	  (completing-read "Previous node (TAB completes): " nodes))
-    (setq up-node (completing-read "Upper node (TAB completes): " nodes))
+	  (Texinfo-nodename-escape
+	   (completing-read "Previous node (TAB completes): " nodes)))
+    (setq up-node (Texinfo-nodename-escape
+		   (completing-read "Upper node (TAB completes): " nodes)))
     (when (and active-mark
 	       (< (mark) (point)))
       (exchange-point-and-mark))
@@ -342,14 +395,26 @@ for @node."
 	      (progn (skip-chars-forward "^,") (forward-char 2))
 	    (throw 'break nil)))))))
 
+(defun Texinfo-arg-nodename (optional &optional prompt definition)
+  "Prompt for a node name completing with known node names.
+OPTIONAL is ignored.
+Use PROMPT as the prompt string.
+If DEFINITION is non-nil, then chosen node name is a node name to be
+added to the list of defined node names. Current implementation
+ignored DEFINITION as the full document is scanned for node names at
+each invocation."
+  (let ((node-name (completing-read (TeX-argument-prompt optional prompt "Key")
+				    (Texinfo-make-node-list))))
+    (insert "{" (Texinfo-nodename-escape node-name) "}" )))
+
 ;; Silence the byte-compiler from warnings for variables and functions declared
 ;; in reftex.
-(eval-when-compile
-  (defvar reftex-section-levels-all)
-  (defvar reftex-level-indent)
-  (defvar reftex-label-menu-flags)
-  (defvar reftex-tables-dirty)
+(defvar reftex-section-levels-all)
+(defvar reftex-level-indent)
+(defvar reftex-label-menu-flags)
+(defvar reftex-tables-dirty)
 
+(eval-when-compile
   (when (fboundp 'declare-function)
     (declare-function reftex-match-string "reftex" (n))
     (declare-function reftex-section-number "reftex-parse" (&optional level star))
@@ -537,6 +602,7 @@ value of `Texinfo-mode-hook'."
   (interactive)
   (kill-all-local-variables)
   (setq TeX-mode-p t)
+  (setq TeX-output-extension (if TeX-PDF-mode "pdf" "dvi"))
   (setq TeX-sentinel-default-function 'TeX-TeX-sentinel)
   ;; Mostly stolen from texinfo.el
   (setq TeX-base-mode-name "Texinfo")
@@ -552,17 +618,16 @@ value of `Texinfo-mode-hook'."
   (set (make-local-variable 'require-final-newline) t)
   (set (make-local-variable 'indent-tabs-mode) nil)
   (set (make-local-variable 'paragraph-separate)
-       (concat "\b\\|^@[a-zA-Z]*[ \n]\\|" paragraph-separate))
+       (concat "\b\\|@[a-zA-Z]*[ \n]\\|" paragraph-separate))
   (set (make-local-variable 'paragraph-start)
-       (concat "\b\\|^@[a-zA-Z]*[ \n]\\|" paragraph-start))
+       (concat "\b\\|@[a-zA-Z]*[ \n]\\|" paragraph-start))
   (set (make-local-variable 'fill-column) 72)
   (set (make-local-variable 'comment-start) "@c ")
   (set (make-local-variable 'comment-start-skip) "@c +\\|@comment +")
   (set (make-local-variable 'comment-use-syntax) nil)
   (set (make-local-variable 'words-include-escapes) t)
-  (if (boundp 'texinfo-imenu-generic-expression)
-      ;; This was introduced in 19.30.
-      (set (make-local-variable 'imenu-generic-expression) texinfo-imenu-generic-expression))
+  (set (make-local-variable 'imenu-generic-expression)
+       texinfo-imenu-generic-expression)
 
   (set (make-local-variable 'font-lock-defaults)
 	;; COMPATIBILITY for Emacs 20
@@ -571,15 +636,16 @@ value of `Texinfo-mode-hook'."
 	      nil nil nil backward-paragraph
 	      (font-lock-syntactic-keywords
 	       . texinfo-font-lock-syntactic-keywords))
-	  '(texinfo-font-lock-keywords t)))
-  (if (not (boundp 'texinfo-section-list))
-      ;; This was included in 19.31.
-      ()
-    (set (make-local-variable 'outline-regexp)
-	 (concat "@\\("
-		 (mapconcat 'car texinfo-section-list "\\>\\|")
-		 "\\>\\)"))
-    (set (make-local-variable 'outline-level) 'texinfo-outline-level))
+	  ;; This is for Emacs >= 23.3, when
+	  ;; `texinfo-font-lock-syntactic-keywords' was removed.
+	  '(texinfo-font-lock-keywords nil nil nil backward-paragraph)))
+
+  ;; Outline settings.
+  (set (make-local-variable 'outline-regexp)
+       (concat "@\\("
+	       (mapconcat 'car texinfo-section-list "\\>\\|")
+	       "\\>\\)"))
+  (set (make-local-variable 'outline-level) 'texinfo-outline-level)
 
   ;; Mostly AUCTeX stuff
   (easy-menu-add Texinfo-mode-menu Texinfo-mode-map)
@@ -604,9 +670,10 @@ value of `Texinfo-mode-hook'."
   (set (make-local-variable 'TeX-font-replace-function) 'TeX-font-replace-macro)
   (set (make-local-variable 'TeX-style-hook-dialect) :texinfo)
 
-  (add-hook 'find-file-hooks (lambda ()
-			       (unless (file-exists-p (buffer-file-name))
-				 (TeX-master-file nil nil t))) nil t)
+  (add-hook 'find-file-hook (lambda ()
+                              (unless (file-exists-p (buffer-file-name))
+                                (TeX-master-file nil nil t)))
+            nil t)
 
   (when (and (boundp 'add-log-current-defun-function)
 	     (fboundp 'texinfo-current-defun-name))
@@ -661,7 +728,10 @@ value of `Texinfo-mode-hook'."
    '("findex" (TeX-arg-literal " ") (TeX-arg-free "Entry"))
    '("footnote" "Text of footnote")
    '("footnotestyle" (TeX-arg-literal " ") (TeX-arg-free "Style"))
-   '("group")
+   '("guillemetleft")
+   '("guillemetright")
+   '("guilsinglleft")
+   '("guilsinglright")
    '("heading" (TeX-arg-literal " ") (TeX-arg-free "Title"))
    ;; XXX: Would be nice with completion.
    '("headings" (TeX-arg-literal " ") (TeX-arg-free "On off single double"))
@@ -692,10 +762,12 @@ value of `Texinfo-mode-hook'."
    '("point" nil)
    '("print")
    '("printindex" (TeX-arg-literal " ") (TeX-arg-free "Index name"))
-   '("pxref" "Node name")
+   '("pxref" (Texinfo-arg-nodename "Node name"))
+   '("quotedblbase")
+   '("quotesinglbase")
    '("r" "Text")
    '("raisesections" 0)
-   '("ref" "Node name")
+   '("ref" (Texinfo-arg-nodename "Node name"))
    '("refill")
    '("result")
    '("samp" "Text")
@@ -742,7 +814,7 @@ value of `Texinfo-mode-hook'."
    '("vindex" (TeX-arg-literal " ") (TeX-arg-free "Entry"))
    '("vskip" (TeX-arg-literal " ") (TeX-arg-free "Amount"))
    '("w" "Text")
-   '("xref" "Node name"))
+   '("xref" (Texinfo-arg-nodename "Node name")))
 
   ;; RefTeX plugging
   (add-hook 'reftex-mode-hook 'Texinfo-reftex-hook)
